#ifndef MFMM_H
#define MFMM_H

#include "fmm.h"
#include "darray.h"


class 	ModifiedFastMarchingMethod : public FastMarchingMethod
	{
	public:	
		enum 	METHOD						//Method for computing (propagating) the count-field:
			{						//
			  AVERAGING,					// - by averaging the counts of already computed neighbours
			  ONE_POINT,					// - by simply propagating the count of the one active point
			  BOUNDARY					// - by finding boundary-point the propagaton came from   
			};  

	
			ModifiedFastMarchingMethod(FIELD<float>*,FLAGS*,FIELD<float>*,int);
						   			//Ctor
		int     execute(int&,int&,float=INFINITY);		//Enh inherited to compute the
									//count field
		void    setScanDir(int);				//Set scanning direction. arg can be 0 or 1, thus generating
									//hopefully totally different 'connection-lines'
		void    setMethod(METHOD);				//Set way in which the count-field is computed
		float	getLength();					//Get boundary-length computed in execute()
									
	protected:
	
		void	add_to_narrowband(int,int,int,int);		//Enh inherited to update 'count'

	private:

		FIELD<float>*	count;					//Maintains the propagated boundary-parametrization
		FIELD<int>*     icount;					//Maintains the propagation of 'from'
		DARRAY<Coord>   from;					//Given a boundary-label, tells where, on the orig. 
									//boundary, it came from         
									//managing the narrowband-counting scheme
		int		scan_dir;				//Used in init_count()	
		METHOD		comp_method;				//Method to compute count-field
		float		length;					//Boundary-length, used for wraparound-distance computations
		
		void		init_count();				//Initializes 'count' from narrowband.
		
		int diffuse();
	};	


inline void ModifiedFastMarchingMethod::setScanDir(int d)
{  scan_dir = d;  }


inline void ModifiedFastMarchingMethod::setMethod(METHOD m)
{  comp_method = m;  }
   
inline float ModifiedFastMarchingMethod::getLength()
{  return length;  }

	
#endif				



