#include "mfmm.h"
#include "flags.h"
#include "dqueue.h"
#include "stack.h"
#include <math.h>
#include <iostream.h>


struct  NewValue {  int i; int j; float value; }; 



ModifiedFastMarchingMethod::ModifiedFastMarchingMethod
			    (FIELD<float>* f_,FLAGS* flags_,FIELD<float>* count_,int N_)
			   :FastMarchingMethod(f_,flags_,N_),count(count_),scan_dir(0),comp_method(BOUNDARY)
{
}



void ModifiedFastMarchingMethod::init_count()
{
   count->size(*f); *count = INFINITY;			
   *icount = INFINITY;					//1. Initialize 'count' by monotonically numbering
   STACK<Coord> s; int i,j; float cc;	    		//   the points in initial narrowband. This will
							//   produce connected-chains of increasingly numbered 
   if(scan_dir==0)					//   points on the connected narrowband segments
   { for(j=0;j<count->dimY();j++)			//   The search for the narrowband points is done from 		
       for(i=0;i<count->dimX();i++) 			//   top-to-bottom in the data, if scan_dir==0, else from
          if (flags->narrowband(i,j)) s.Push(Coord(i,j)); // bottom to top. The found points are put in a container.
   }
   else
   { for(j=count->dimY()-1;j>=0;j--)			
       for(i=0;i<count->dimX();i++) 
          if (flags->narrowband(i,j)) s.Push(Coord(i,j));
   }

   length = 0; 						//!!!
   int  C = 0;						
   const int INFINITY_2 = int(INFINITY/2);		//2. Now that we collected all narrowband points,
   for(float k=0;s.Count();)				//we start numbering them in the 'count' field.	
   {
      Coord pt = s.Pop(); 				//Pick a point - test whether it is already numbered
      if (count->value(pt.i,pt.j)<INFINITY_2) continue;

      int ii,jj,di,dj,ci=0,cj=0; cc = int(INFINITY);	//Point pt not numbered
      for(dj=-1;dj<2;dj++)				//Find min-numbered-neighbour of pt
	for(di=-1;di<2;di++)
	{  
	   ii=pt.i+di; jj=pt.j+dj; 
   	   if (ii<0 || ii>=count->dimX() || jj<0 || jj>=count->dimY() ||
   	      !flags->narrowband(ii,jj)) continue;         
	   if (count->value(ii,jj)>=cc) continue;
	   ci = ii; cj = jj; cc = count->value(ii,jj);
	}

      float c = (cc>INFINITY_2)? k+1 : cc + sqrt((pt.i-ci)*(pt.i-ci)+(pt.j-cj)*(pt.j-cj));
      count->value(pt.i,pt.j)  = c;
      icount->value(pt.i,pt.j) = C; 
      from[C] = pt; C++;				//Register all boundary-pts in from[]	

      if (c>length) length=c;				//Determine length of boundary, used for wraparound-distance-computations 
      if (k<c) k=c;

      for(dj=-1;dj<2;dj++)				
	for(di=-1;di<2;di++)
	{ if (di==0 && dj==0) continue;
	  ii=pt.i+di; jj=pt.j+dj; 
   	  if (ii<0 || ii>=count->dimX() || jj<0 || jj>=count->dimY() ||
   	     count->value(ii,jj)<INFINITY_2 || !flags->narrowband(ii,jj)) 
	     continue;  		
   	  s.Push(Coord(ii,jj));
        }	
   }
}



int ModifiedFastMarchingMethod::execute(int& negd_, int& nextr_, float maxf_)
{
   from.Init(flags->initialContourLength());		//Allocate memory for from[]
   icount = new FIELD<int>(f->dimX(),f->dimY());	//Allocate memory for icount[][]	

   init_count();					//1. Initialize the 'count' field on boundary.

   int ret = FastMarchingMethod::execute(negd_,nextr_,maxf_);
 							//2. Call inherited fast-marching-method that will
							//   do all the evolution job...

   from.Init(0);					//Ready with from[][], shrink to release memory
   delete icount;					//Ready with icount
   return ret;
}




void ModifiedFastMarchingMethod::add_to_narrowband(int i,int j,int active_i,int active_j)
{
	FastMarchingMethod::add_to_narrowband(i,j,active_i,active_j);

	if (comp_method==AVERAGING || comp_method==BOUNDARY)//AVERAGING/BOUNDARY COMPUTATION METHODS
	{ 							//
		float m=INFINITY,dmin=INFINITY,M=-m,INFINITY_2 = INFINITY/2; 
		float cv=0; int cc=0,pt_i,pt_j;		        //    


		// Laat (pt_i,pt_j) de pixel zijn met een origin op de rand
		for(int ii=i-1;ii<i+2;ii++)			//  
		{
			for(int jj=j-1;jj<j+2;jj++)
			{
				if (flags->alive(ii,jj) && count->value(ii,jj)<INFINITY_2)
				{
					float v = count->value(ii,jj);
					int   V = icount->value(ii,jj);		//Find where (ii,jj) came from on the initial boundary
					Coord c = from[V];
					int   d = (c.i-i)*(c.i-i)+(c.j-j)*(c.j-j);
					if (d<dmin)					
					{  dmin=d; pt_i=ii; pt_j=jj; }		//Find boundary-point closest to (ii,jj)
					if (m>v) m=v; 
					if (M<v) M=v;
					cv += v; 
					cc++;
				}
			}
		}

		// original:
		count->value(i,j) = 
			(comp_method==BOUNDARY) ? 
				count->value(pt_i,pt_j)
				
				// { comp_method == AVERAGING }
				: (M-m<10) ? cv/cc 
				: count->value(active_i,active_j);

		icount->value(i,j) = icount->value(pt_i,pt_j);
						//set the averaged value of ALIVE-nbs' counts having
						//valid (computed) count-values, if we're not smoothing out a 
								//sharp count-discontinuity
	}
    else 
    if (comp_method==ONE_POINT)				//1-POINT COMPUTATION METHOD
		count->value(i,j) = count->value(active_i,active_j);
}




int ModifiedFastMarchingMethod::diffuse()
{
    static NewValue newp[20]; 

    //*** 1. FIND MIN-POINT IN NARROWBAND WITH LOWEST VALUE
    int min_i,min_j;
    std::multimap<float,Coord>::iterator it=map.begin();
    if (it==map.end()) return 0;

    min_i  = (*it).second.i;
    min_j  = (*it).second.j;
    map.erase(it);					//erase point from 'map', since we'll make it alive in step 2

    //*** 2. MAKE MIN-POINT ALIVE
    flags->value(min_i,min_j) = FLAGS::ALIVE;		
    if (f->value(min_i,min_j)>=maxf) return 1;		//stop evolution if we reached the user-prescribed threshold
   
    //*** 3. FIND ALL ITS STILL-TO-BE-UPDATED NEIGHBOURS
    Coord nbs[4]; int nn = 0;
    tag_nbs(min_i-1,min_j,min_i,min_j,nbs,nn);
    tag_nbs(min_i+1,min_j,min_i,min_j,nbs,nn);
    tag_nbs(min_i,min_j-1,min_i,min_j,nbs,nn);
    tag_nbs(min_i,min_j+1,min_i,min_j,nbs,nn);

    if (!nn) 						//no more alive-neighbous of point (min_i,min_j) found,
    { 							//so it should be an extremum point...
	flags->value(min_i,min_j) = FLAGS::EXTREMUM;
        nextr++;
	return 1;	
    }

    //*** 4. UPDATE VALUES OF NEIGHBOURS OF MIN-POINT
    NewValue* nnewp = newp;				//start updating neighbours. Their new values will be saved
    for(nn--;nn>=0;nn--)				//in nnewp[] and pasted back in 'f' at the update end.
    {
	int i = nbs[nn].i, j = nbs[nn].j;

	float vi_1j = f->value(i-1,j),     vijx1 = f->value(i,j+1);
	float vix1j = f->value(i+1,j),     vij_1 = f->value(i,j-1);
	int   fi_1j = flags->value(i-1,j), fijx1 = flags->value(i,j+1);
	int   fix1j = flags->value(i+1,j), fij_1 = flags->value(i,j-1);

	float sol = INFINITY; 
	solve(fi_1j,fij_1,vi_1j,vij_1,sol);
	solve(fix1j,fij_1,vix1j,vij_1,sol);
	solve(fi_1j,fijx1,vi_1j,vijx1,sol); 
	solve(fix1j,fijx1,vix1j,vijx1,sol); 

	if (sol < INFINITY/2) 
        { nnewp->i = i; nnewp->j = j; nnewp->value = sol; nnewp++; }
    }

    //***5. Write updated values back in field.
    for(nnewp--;nnewp>=newp;nnewp--)				//for all updated neighbours:
    {
       map.erase(ptrs.value(nnewp->i,nnewp->j));		//remove the neighbour's entry from the sorted map...
       std::multimap<float,Coord>::value_type v(nnewp->value,Coord(nnewp->i,nnewp->j));
       ptrs.value(nnewp->i,nnewp->j) = map.insert(v);		//...and insert it back since its field-value changed		
       f->value(nnewp->i,nnewp->j) = nnewp->value;		//update the field too!
   }
   return 1;
}


