#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <iostream.h>
#include "field.h"
#include "flags.h"
#include <GL/glut.h>
#include "genrl.h"
#include "io.h"
#include "mfmm.h"



//----------------------------------------------------------------

FIELD<float>* do_one_pass(FIELD<float>*,FLAGS*,int,int,ModifiedFastMarchingMethod::METHOD,float&);
void comp_grad(FLAGS*,FIELD<float>*,FIELD<float>*);
void comp_grad2(FLAGS*,FIELD<float>*,FIELD<float>*,FIELD<float>*);
void postprocess(FIELD<float>*,FIELD<float>*,float);
void key_cb(unsigned char,int,int);

float length;
float sk_lev;
FIELD<float> *f,*grad;

inline float distance(int cd)
//Computes distance along boundary in a wrap-around fashion...
{
   return min(fabs(cd),fabs(length-fabs(cd)));
}

int xx=-1;


int main(int argc,char* argv[])
{
   glutInit(&argc, argv);
   char inpf[200],*input;
   argc--;argv++;					//Skip program name arg
   cout<<"DISCRETE SKELETON 2D"<<endl<<endl;

   if (argc<1)
   {
      cout<<"Input file:"<<endl;
      cin>>inpf; input = inpf;
   }
   else input    = argv[0];

   float k = -1;	 			    	//Threshold for clusters (def)
   int   N = 1000000;					//Max # fast marching method iterations
   sk_lev  = 10;	    				//Skeleton threshold (default=30 pixels)
   int  twopass = 1;					//Using 2-pass or 1-pass method for boundary treatment 
   ModifiedFastMarchingMethod::METHOD			//Method used to compute count-field in the MFMM algo
        meth = ModifiedFastMarchingMethod::BOUNDARY;
//        meth = ModifiedFastMarchingMethod::AVERAGING;

   for(;argc;)						//Process cmdline args
   {
	char* opt = argv[0]+1; argc--;argv++;
        char* val = argv[0];

	if (opt[0]=='t') 				// -t <threshold>
	{  k = atof(val); argc--;argv++;  }
        else if (opt[0]=='N')				// -N <maxiter>		
        {  N = atoi(val); argc--;argv++;  } 
	else if (opt[0]=='s')				// -s <skeleton_level>
	{  sk_lev = atof(val); argc--;argv++;  }
        else if (opt[0]=='p')				// -p
        {  twopass = 0;  }
	else if (opt[0]=='m')				// -m <method>
        {
	   meth = (atoi(val)==0)? ModifiedFastMarchingMethod::BOUNDARY : // Dennie: AFMM start
	          (atoi(val)==1)? ModifiedFastMarchingMethod::AVERAGING : // Dennie: AFMM oud
			          ModifiedFastMarchingMethod::ONE_POINT;
	   argc--;argv++;
        }   
        else if (opt[0]=='x')
        {  xx = atoi(val); argc--;argv++; }
   }


   char  inp[100]; strcpy(inp,input);

   f = FIELD<float>::read(inp);		        //Read scalar field input
   if (!f) { cout<<"Can not open file: "<<inp<<endl; return 1; }
 
   FLAGS*        flags = new FLAGS(*f,k); 
   FIELD<float>* cnt0  = do_one_pass(f,flags,N,0,meth,length);
   FIELD<float>* cnt1  = (twopass)? do_one_pass(f,flags,N,1,meth,length) : 0;
   grad  = new FIELD<float>(f->dimX(),f->dimY());
   if (twopass)  comp_grad2(flags,cnt0,cnt1,grad);
   else          comp_grad(flags,cnt0,grad);
  
   postprocess(grad,f,sk_lev);

   cout<<"Use +/- keys in graphics window to select the simplification level"<<endl;
 
   glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
   glutInitWindowSize(f->dimX(),f->dimY());
   
   f->display();
   glutKeyboardFunc(key_cb);
   glutMainLoop();
   
   delete f;
   delete flags;
   delete cnt0;
   delete cnt1;
   delete grad;
   return 0;
}



FIELD<float>* do_one_pass(FIELD<float>* fi,FLAGS* flagsi,int N,int dir,ModifiedFastMarchingMethod::METHOD meth,float& length)
{
   FIELD<float>*      f = new FIELD<float>(*fi);	//Copy input field 
   FLAGS*   	  flags = new FLAGS(*flagsi);		//Copy flags field
   FIELD<float>*  count = new FIELD<float>;

   ModifiedFastMarchingMethod fmm(f,flags,count,N);	//Create fast marching method obj
   fmm.setScanDir(dir);
   fmm.setMethod(meth);

   int nfail,nextr;
   fmm.execute(nfail,nextr);
   length = fmm.getLength();

   delete flags;
   delete f;
   return count;
}




void comp_grad2_new(FLAGS* forig,FIELD<float>* cnt1,FIELD<float>* cnt2,FIELD<float>* grad)
//Gradient computation using 2-pass method (i.e. grad computed out of 2 fields)
{
   int i,j; *grad = 0;

   for(j=1;j<grad->dimY()-1;j++)			//Compute grad in a special way, i.e. on a 2-pixel 
     for(i=1;i<grad->dimX()-1;i++) 			//neighbourhood - this ensures pixel-size skeletons!	
     {
	if (forig->value(i,j)==FLAGS::ALIVE || forig->value(i,j)==FLAGS::NARROW_BAND)
	{ grad->value(i,j) = 0; continue; }

        float u[4]; float uM; int k;
   	u[0] = cnt1->value(i,j) - cnt1->value(i+1,j);
   	u[1] = cnt1->value(i,j) - cnt1->value(i,j+1);
        u[2] = cnt1->value(i,j) - cnt1->value(i-1,j);
        u[3] = cnt1->value(i,j) - cnt1->value(i,j-1); 
        uM=u[0]; for(k=1;k<4;k++) if (u[k]>uM) uM = u[k];
	float g1  = max(uM,0);

   	u[0] = cnt2->value(i,j) - cnt2->value(i+1,j);
   	u[1] = cnt2->value(i,j) - cnt2->value(i,j+1);
        u[2] = cnt2->value(i,j) - cnt2->value(i-1,j);
        u[3] = cnt2->value(i,j) - cnt2->value(i,j-1); 
        uM=u[0]; for(k=1;k<4;k++) if (u[k]>uM) uM = u[k];
	float g2  = max(uM,0);

	grad->value(i,j) = min(g1,g2);
     }
}


void comp_grad2(FLAGS* forig,FIELD<float>* cnt1,FIELD<float>* cnt2,FIELD<float>* grad)
//Gradient computation using 2-pass method (i.e. grad computed out of 2 fields)
{
   int i,j;

   for(j=0;j<grad->dimY();j++)				//Compute grad in a special way, i.e. on a 2-pixel 
     for(i=0;i<grad->dimX();i++) 			//neighbourhood - this ensures pixel-size skeletons!	
     {
   	float ux1 = cnt1->value(i+1,j) - cnt1->value(i,j);
   	float uy1 = cnt1->value(i,j+1) - cnt1->value(i,j);
	float g1  = max(fabs(ux1),fabs(uy1));
   	float ux2 = cnt2->value(i+1,j) - cnt2->value(i,j);
   	float uy2 = cnt2->value(i,j+1) - cnt2->value(i,j);
	float g2  = max(fabs(ux2),fabs(uy2));
	grad->value(i,j) = min(g1,g2);
     }
}


void comp_grad_new(FLAGS* forig,FIELD<float>* cnt,FIELD<float>* grad)
//Gradient computation using 1-pass method (i.e. grad computed out of 1 field)
{
   int i,j;
   for(j=0;j<grad->dimY();j++)				//Compute grad in a special way, i.e. on a 2-pixel 
     for(i=0;i<grad->dimX();i++) 			//neighbourhood - this ensures pixel-size skeletons!	
     {
	if (forig->value(i,j)==FLAGS::ALIVE || forig->value(i,j)==FLAGS::NARROW_BAND)
	{ grad->value(i,j) = 0; continue; }

        float u[4]; float uM; int k;
   	u[0] = cnt->value(i,j) - cnt->value(i+1,j);
   	u[1] = cnt->value(i,j) - cnt->value(i,j+1);
        u[2] = cnt->value(i,j) - cnt->value(i-1,j);
        u[3] = cnt->value(i,j) - cnt->value(i,j-1); 
        uM=u[0]; for(k=1;k<4;k++) if (u[k]>uM) uM = u[k];
	if (uM<0) uM=0;
        grad->value(i,j) = (uM<length/2)? uM : length - uM;
     }
}


void comp_grad(FLAGS* forig,FIELD<float>* cnt,FIELD<float>* grad)
//Gradient computation using 1-pass method (i.e. grad computed out of 1 field)
{
   int i,j;
   for(j=0;j<grad->dimY();j++)				//Compute grad in a special way, i.e. on a 2-pixel 
     for(i=0;i<grad->dimX();i++) 			//neighbourhood - this ensures pixel-size skeletons!	
     {
   	float ux = cnt->value(i+1,j) - cnt->value(i,j);
   	float uy = cnt->value(i,j+1) - cnt->value(i,j);
	grad->value(i,j) = max(distance(ux),distance(uy));
     }
}


void postprocess(FIELD<float>* grad,FIELD<float>* skel,float level)
//Simple reduction of gradient to binary skeleton via thresholding
{
   for(int j=0;j<grad->dimY();j++)			//Threshold 'grad' to get real skeleton
     for(int i=0;i<grad->dimX();i++)
     {
	   float g = grad->value(i,j);
	   if (g>INFINITY/2) g=0.5; else
	   if (g>level) g = 1; else g = 0;
	   skel->value(i,j)=g;
     } 
}





void key_cb(unsigned char ch,int,int)
{
    switch(ch)
    {
    case '=': sk_lev++; break;
    case '-': sk_lev--; break;
    case '_': sk_lev-=10; break;
    case '+': sk_lev+=10; break;
    }
    if (sk_lev<2) sk_lev = 2;
    postprocess(grad,f,sk_lev);
    glutPostRedisplay();
}