/*
 * HnTimesObj.cc
 * Copyright (C) 1999 Norio Katayama
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 *
 * 02/02/99 Norio KATAYAMA
 * $Id: HnTimesObj.cc,v 1.3 2000/12/01 06:34:51 katayama Exp $
 */

#include <stdio.h>
#include <unistd.h>
#include "HnSRTree/config.h"

#ifdef HAVE_GETRUSAGE
#include <sys/resource.h>
#else
#include <sys/times.h>
#include <limits.h>
#endif

#include "HnSRTree/HnTimes.hh"
#include "HnSRTree/HnTimesObj.hh"

HnTimesObj::HnTimesObj(void)
{
    rtime.tv_sec = 0;
    rtime.tv_usec = 0;

    utime.tv_sec = 0;
    utime.tv_usec = 0;

    stime.tv_sec = 0;
    stime.tv_usec = 0;
}

void
HnTimesObj::setCurrentTimes(void)
{
    gettimeofday(&rtime, NULL);

#ifdef HAVE_GETRUSAGE
    {
	struct rusage rusage;

	if(getrusage(RUSAGE_SELF, &rusage) < 0) {
	    perror("getrusage");
	    exit(1);
	}

	utime = rusage.ru_utime;
	stime = rusage.ru_stime;
    }
#else
    {
	struct tms tms;

	if(times(&tms) == -1) {
	    perror("times");
	    exit(1);
	}

	utime.tv_sec = tms.tms_utime / CLK_TCK;
	utime.tv_usec = (tms.tms_utime % CLK_TCK) * 1000000 / CLK_TCK;
	stime.tv_sec = tms.tms_stime / CLK_TCK;
	stime.tv_usec = (tms.tms_stime % CLK_TCK) * 1000000 / CLK_TCK;
    }
#endif
}

static void
addTimeval(struct timeval *tp3,
	   const struct timeval *tp1, const struct timeval *tp2)
{
    tp3->tv_sec  = tp1->tv_sec + tp2->tv_sec;
    tp3->tv_usec = tp1->tv_usec + tp2->tv_usec;

    if (tp3->tv_usec >= 1000000) {
	tp3->tv_sec ++;
	tp3->tv_usec -= 1000000;
    }
}

HnTimes
HnTimesObj::add(const HnTimes &times) const
{
    HnTimes newTimes = new_HnTimes();

    addTimeval(&newTimes.getObject()->rtime,
	       &rtime, &times.getObject()->rtime);
    addTimeval(&newTimes.getObject()->utime,
	       &utime, &times.getObject()->utime);
    addTimeval(&newTimes.getObject()->stime,
	       &stime, &times.getObject()->stime);

    return newTimes;
}

static void
subtractTimeval(struct timeval *tp3,
		const struct timeval *tp1, const struct timeval *tp2)
{
    tp3->tv_sec  = tp1->tv_sec - tp2->tv_sec;

    if (tp1->tv_usec >= tp2->tv_usec) {
	tp3->tv_usec = tp1->tv_usec - tp2->tv_usec;
    }
    else {
	tp3->tv_sec --;
	tp3->tv_usec = 1000000 + tp1->tv_usec - tp2->tv_usec;
    }
}

HnTimes
HnTimesObj::subtract(const HnTimes &times) const
{
    HnTimes newTimes = new_HnTimes();

    subtractTimeval(&newTimes.getObject()->rtime,
		    &rtime, &times.getObject()->rtime);
    subtractTimeval(&newTimes.getObject()->utime,
		    &utime, &times.getObject()->utime);
    subtractTimeval(&newTimes.getObject()->stime,
		    &stime, &times.getObject()->stime);

    return newTimes;
}

double
HnTimesObj::getRealTime(void) const
{
    return ((double)rtime.tv_sec + (double)rtime.tv_usec / 1000000);
}
    
double
HnTimesObj::getUserTime(void) const
{
    return ((double)utime.tv_sec + (double)utime.tv_usec / 1000000);
}

double
HnTimesObj::getSystemTime(void) const
{
    return ((double)stime.tv_sec + (double)stime.tv_usec / 1000000);
}

double
HnTimesObj::getCPUTime(void) const
{
    return (getUserTime() + getSystemTime());
}
