/*
 * HnSRTreeFileSt.h
 * Copyright (C) 1999 Norio Katayama
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 *
 * 07/27/1999 Norio KATAYAMA
 * $Id: HnSRTreeFileSt.h,v 1.4 2000/06/10 11:19:55 katayama Exp $
 */

#ifndef _HnSRTreeFileSt_h
#define _HnSRTreeFileSt_h

#include <stdio.h>

#include "HnSRTree/HnAbort.h"
#include "HnSRTree/HnPointSt.h"
#include "HnSRTree/HnDataItemSt.h"
#include "HnSRTree/HnRectSt.h"
#include "HnSRTree/HnPropertiesSt.h"

#include "HnSRTree/HnSRTreeProto.h"
#include "HnSRTree/HnSRTreeProfileSt.h"

/*
 * Structure
 */

typedef struct {
    void *opaque;
} HnSRTreeFileSt;

/*
 * Functions
 */

#ifdef __cplusplus
extern "C" {
#endif

HnSRTreeFileSt *HnSRTreeFileSt_create(const char *path,
				      int dimension, int dataItemSize, 
				      const HnPropertiesSt *properties);
HnSRTreeFileSt *HnSRTreeFileSt_open(const char *path, const char *mode);
HnSRTreeFileSt *HnSRTreeFileSt_build(const char *path,
				     int dimension, int dataItemSize,
				     const HnPointVectorSt *points,
				     const HnDataItemVectorSt *dataItems,
				     const HnPropertiesSt *properties);
void HnSRTreeFileSt_close(HnSRTreeFileSt *file);

void HnSRTreeFileSt_store(HnSRTreeFileSt *file,
			  const HnPointSt *point,
			  const HnDataItemSt *dataItem);
void HnSRTreeFileSt_remove(HnSRTreeFileSt *file,
			   const HnPointSt *point,
			   const HnDataItemSt *dataItem);

void HnSRTreeFileSt_getFirst(HnSRTreeFileSt *file,
			     const HnRectSt *queryRect,
			     HnPointSt **point_return,
			     HnDataItemSt **dataItem_return);
void HnSRTreeFileSt_getNext(HnSRTreeFileSt *file,
			    HnPointSt **point_return,
			    HnDataItemSt **dataItem_return);

void HnSRTreeFileSt_getInRect(HnSRTreeFileSt *file,
			      const HnRectSt *queryRect,
			      HnPointVectorSt **points_return,
			      HnDataItemVectorSt **dataItems_return);

void HnSRTreeFileSt_getNeighbors(HnSRTreeFileSt *file,
				 const HnPointSt *queryPoint, int maxCount,
				 HnPointVectorSt **points_return,
				 HnDataItemVectorSt **dataItems_return);
void HnSRTreeFileSt_getColoredNeighbors(HnSRTreeFileSt *file,
					const HnPointVectorSt *queryPoints,
					int maxCount,
					HnPointVectorSt **points_return,
					HnDataItemVectorSt **dataItems_return,
					HnSRTreeCompColorsFunc *
					compColorsFunc);

int HnSRTreeFileSt_getDimension(HnSRTreeFileSt *file);
int HnSRTreeFileSt_getBlockSize(HnSRTreeFileSt *file);
int HnSRTreeFileSt_getDataItemSize(HnSRTreeFileSt *file);
int HnSRTreeFileSt_getHeight(HnSRTreeFileSt *file);

void HnSRTreeFileSt_check(HnSRTreeFileSt *file);
void HnSRTreeFileSt_print(HnSRTreeFileSt *file, HnBool verbose);

void HnSRTreeFileSt_resetProfile(HnSRTreeFileSt *file);
void HnSRTreeFileSt_copyProfileInto(HnSRTreeFileSt *file,
				    HnSRTreeProfileSt *profile);

HnBool HnSRTreeFileSt_dumpToFile(HnSRTreeFileSt *file, const char *fileName);
HnBool HnSRTreeFileSt_dumpToFileStream(HnSRTreeFileSt *file, FILE *fp);

void HnSRTreeFileSt_setProperties(HnSRTreeFileSt *file,
				  const HnPropertiesSt *properties);
void HnSRTreeFileSt_getProperties(HnSRTreeFileSt *file,
				  HnPropertiesSt *properties_return);

void HnSRTreeFileSt_setDebug(HnBool debug);

#ifdef __cplusplus
};
#endif

#endif /* _HnSRTreeFileSt_h */
